#include <cmath>
#include <conio.h>

#include <tinyxml.h>
#include <string>
#include <vector>
using std::string;
using std::vector;

struct Stat {
	static void increase(unsigned ID, float& val, float weight, float modifier) {
		float capFactor = (1.0f-val);
		float factor = pow(.5f*((modifier-val)+1.0f), 2.0f);
		if (factor>0) val+=weight*capFactor*factor*learnFactors[ID];
	}

	static void loadStats() {
		TiXmlDocument xml("stats.xml");
		xml.LoadFile();
		TiXmlElement* e = xml.RootElement()->FirstChildElement("stat");
		while (e) {
			Stat::names.push_back(e->Attribute("name"));
			Stat::learnFactors.push_back(0);
			e->QueryFloatAttribute("learnFactor", &Stat::learnFactors.back());
			e=e->NextSiblingElement("stat");
		}
	}
	static vector<float> learnFactors;
	static vector<string> names;
};

vector<float> Stat::learnFactors;
vector<string> Stat::names;

struct Action {
	bool failed;
	vector<float> weight;
	vector<unsigned> stat;
	
	static float modifierFactor;
	static vector<Action> actions;
	static void loadActions() {
		TiXmlDocument xml("actions.xml");
		xml.LoadFile();
		TiXmlElement* e = xml.RootElement()->FirstChildElement("action");
		while (e) {
			TiXmlElement* s = e->FirstChildElement("stat");
			Action na;
			na.failed=0;
			while (s) {
				const char* n = s->Attribute("name");
				for (unsigned i=0; i<Stat::names.size(); i++) {
					if (Stat::names[i]==n) {
						na.stat.push_back(i);
						na.weight.push_back(0);
						s->QueryFloatAttribute("weight", &na.weight.back());
						break;
					}
				}
				s=s->NextSiblingElement("stat");
			}
			Action::actions.push_back(na);
			e=e->NextSiblingElement("action");
		}
		e = xml.RootElement()->FirstChildElement("factor");
		if (e) e->QueryFloatAttribute("modifier", &Action::modifierFactor);
	}
};

vector<Action> Action::actions;
float Action::modifierFactor=.5f;

struct Character {
	float stats[5];
	unsigned action;

	float doAction(Action& a, float difficulty);

	Character(const char* file) : action(0) {
		TiXmlDocument xml(file);
		xml.LoadFile();
		TiXmlElement* e = xml.RootElement()->FirstChildElement("stats");
		if (e) {
			TiXmlAttribute* a = e->FirstAttribute();
			while (a) {
				for (unsigned i=0; i<Stat::names.size(); i++) {
					if (Stat::names[i]==a->Name()) {
						double tmp;
						a->QueryDoubleValue(&tmp);
						stats[i]=(float)tmp;
						break;
					}
				}
				a = a->Next();
			}
		}
	}
};

int increases=0;

float Character::doAction(Action& a, float difficulty) {
	float effDiff = 0;
	for (unsigned i=0; i<a.stat.size(); i++) effDiff += a.weight[i] * stats[a.stat[i]];
	float res=.5f + Action::modifierFactor*(effDiff-difficulty) - 0.01f*(rand()%101);

	if (a.failed && res>=0) {
		increases++;
		for (unsigned i=0; i<a.stat.size(); i++) Stat::increase(a.stat[i], stats[a.stat[i]], a.weight[i], difficulty);
	}
	a.failed=(res<0);
	return res;
}

int main() {
	Stat::loadStats();
	Action::loadActions();
	Character test("character.xml");

	int shotsTaken[2]={0}, hit=0, miss=0;
	float difficulties[6]={0.0f, .2f, .4f, .6f, .8f, 1.0f};

	for (;;) {
		for (int i=0; i<5; i++) printf("%s: %.2f\n", Stat::names[i].c_str(), test.stats[i]*100.0f);
		printf("\nEquipped: %s (fired %d times)\n", (test.action==0?"pistol":"rifle"), shotsTaken[test.action]);
		printf("\nStat increases: %d (hit/miss: %d/%d)\n", increases, hit, miss);
		printf("\n'1'-'6': shoot (difficulty 0%%-100%%), 'r': equip rifle, 'p': equip pistol\n");
			
		char c=_getch();
		if (c<='6') {
			float diff = difficulties[c-'1'];
			float r = test.doAction(Action::actions[test.action], diff);
			printf("\nShooting (%.0f%%): ", diff*100.0f);
			if (r>=0) {printf("Hit\n"); hit++;}
			else {printf("Miss\n"); miss++;}
			shotsTaken[test.action]++;
		}
		if (c=='p') test.action=0;
		if (c=='r') test.action=1;
		if (c=='q') break;
		printf("\n\n\n");
	}
	return 0;
}